@@ -1,13 +1,15 @@
+from typing import Collection
+
 import numpy as np
 
 from copulae.copula import BaseCopula, Summary
 from copulae.stats import random_uniform
 from copulae.types import Array
-from copulae.utility import array_io
+from copulae.utility.annotations import *
 
 
 class IndepCopula(BaseCopula[int]):
-    def __init__(self, dim=2):
+    def __init__(self, dim=2, fields: Collection[str] = None):
         r"""
         The Independence copula is the copula that results from a dependency structure in which each individual
         variable is independent of each other. It has no parameters and is defined as
@@ -21,16 +23,18 @@ class IndepCopula(BaseCopula[int]):
         dim: int, optional
             The dimension of the copula
         """
-        self._dim = dim
-        self._name = 'Independent'
-        self.init_validate()
+        super().__init__(dim, "Independent")
+        if fields is not None:
+            self._columns = list(map(str, fields))
+            assert len(fields) == dim, "number of fields must match copula dimension"
 
-    @array_io
-    def cdf(self, x: Array, log=False) -> np.ndarray:
+    @validate_data_dim({"x": [1, 2]})
+    @shape_first_input_to_cop_dim
+    @squeeze_output
+    def cdf(self, x: Array, log=False):
         return np.log(x).sum(1) if log else x.prod(1)
 
-    def fit(self, data: np.ndarray, x0: np.ndarray = None, method='mpl', verbose=1, optim_options: dict = None,
-            ties='average'):
+    def fit(self, data, x0=None, method='ml', verbose=1, optim_options=None, ties='average'):
         print('Fitting not required for Independent Copula')
         return self
 
@@ -38,11 +42,14 @@ class IndepCopula(BaseCopula[int]):
     def params(self):
         return self.dim
 
-    @array_io
-    def pdf(self, x: Array, log=False) -> np.ndarray:
+    @validate_data_dim({"x": [1, 2]})
+    @shape_first_input_to_cop_dim
+    @squeeze_output
+    def pdf(self, x: Array, log=False):
         return np.repeat(0 if log else 1, len(x))
 
-    def random(self, n: int, seed: int = None) -> np.ndarray:
+    @cast_output
+    def random(self, n: int, seed: int = None):
         return random_uniform(n, self.dim, seed)
 
     def summary(self):
